/* --------------------------------------------------------------- */
/* (C) Copyright 2006,2007,                                        */
/* International Business Machines Corporation                     */
/* All Rights Reserved.                                            */
/*                                                                 */
/* THIS PROGRAM IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC   */
/* LICENSE VERSION 1 ("AGREEMENT"). ANY USE, REPRODUCTION OR       */
/* DISTRIBUTION OF THE PROGRAM CONSTITUTES RECIPIENT'S ACCEPTANCE  */
/* OF THIS AGREEMENT.                                              */
/*                                                                 */
/* A copy of the Agreement accompanies this distribution, or see   */
/* <http://www.ibm.com/developerworks/library/os-cpl.html>.        */
/* --------------------------------------------------------------- */
/* PROLOG END TAG zYx                                              */


/*********************************************************************
*                                                                    *
* Encrypt the .text section, starting at the specified offset and    *
* with the specified size. Correct the offset and size for the       *
* required cipher block size.                                        *
*                                                                    *
*********************************************************************/

#include <stddef.h>		 // NULL, other ANSI-C types
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>

#include "buildSecureApp.h"

#define XOR_BLOCK_SIZE ENCRYPTION_KEY_SIZE

int encryptSection(MyElf32_Shdr* encryptShdr, unsigned char* data, Elf32_Word size, EncryptInfo* encryptInfo)
{
	unsigned char*	dataptr;
	unsigned char*	encryptedKey = NULL;
	int				encryptedKeySize = 0;
	Elf32_Off		endOffset;
	int				i;
	Elf32_Off		offset;
	int				rc = 0;
	Elf32_Off		startOffset;
	unsigned char*	xorKey = encryptInfo->encryptKey;

	if (encryptInfo->encryptKeySize != ENCRYPTION_KEY_SIZE) {
		PRINTF(("Failure: internal error\n"));
		rc = -901;
		goto err01;
	}
	offset = encryptInfo->encryptOffset - encryptShdr->sh_addr;
	startOffset = ROUND_DOWN(offset,XOR_BLOCK_SIZE);
	endOffset = offset + encryptInfo->encryptSize;
	endOffset = ROUND_UP(endOffset,XOR_BLOCK_SIZE);
	if ((startOffset < 0) || (endOffset > size)) {
		PRINTF(("Failure: internal error\n"));
		rc = -902;
		goto err01;
	}

	/* XOR the "key" with the data */
	for (dataptr=data+startOffset; dataptr<data+endOffset; dataptr+=16) {
		for (i=0; i<16; i++) {
			*(dataptr+i) ^= *(xorKey+i);
		}
	}

	/* Copy the xor "key" to save in the code image */
	encryptedKeySize = encryptInfo->encryptKeySize;
	if ((encryptedKey = malloc(encryptedKeySize)) == NULL) {
		PRINTF(("Failure: unable to malloc %d bytes\n",encryptedKeySize));
		rc = -910;
		goto err02;
	}
	memcpy(encryptedKey,xorKey,encryptedKeySize);
	encryptInfo->encryptedKey = encryptedKey;
	encryptInfo->encryptedKeySize = encryptedKeySize;

	encryptInfo->encryptOffset = startOffset;
	encryptInfo->encryptSize = endOffset - startOffset;

err02:
err01:
	return rc;
}
